@@ -1,6 +1,3 @@
-# Copyright 2015
-# All Rights Reserved.
-#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -15,9 +12,10 @@
 
 from oslo_config import cfg
 
-image_feature_opts = [
-    cfg.BoolOpt('enforce_scope',
-                default=False,
-                help='Does the image service enforce scope and user '
-                     'scope-aware policies?'),
-]
+glance_api_group = cfg.OptGroup(name="glance_api",
+                                title="Glance API Service Options")
+
+GlanceAPIGroup = [cfg.BoolOpt("system_scope_supported",
+                              default=False,
+                              help="Whether or not system scope "
+                                   "is supported")]
